import type_enforced


def run_tests(fn, success):
    try:
        fn(a='a', b='b', c='c') # No Error
        fn(a='a') # No Error
        fn('a') # No Error
        fn('a', b='b') # No Error
        fn('a', 'b') # No Error
        try:
            fn(a='a', b=2, c='c') # Error (b can only accept str)
            print('a')
            success=False
        except:
            pass
        try:
            fn('a', b=2) # Error (b can only accept str)
            print('b')
            success=False
        except:
            pass
        try:
            fn('a', 2) # Error (b can only accept str)
            success=False
        except:
            pass
    except:
        success=False
    return success


success=True

@type_enforced.Enforcer
def fn1(a, b: str = 'b', c=None):
    pass

@type_enforced.Enforcer
def fn2(a, b: str = 'b', c=None, **kwargs):
    pass

@type_enforced.Enforcer
def fn3(a, *args, b: str = 'b', c=None):
    pass

@type_enforced.Enforcer
def fn4(a, *args, b: str = 'b', c=None, **kwargs):
    pass

success = run_tests(fn1, success)
success = run_tests(fn2, success)
success = run_tests(fn3, success)
success = run_tests(fn4, success)

if success:
    print('test_fn_6.py passed')
else:
    print('test_fn_6.py failed')


import type_enforced

@type_enforced.Enforcer
def my_fn(a: int , b: [int, str]=2, c: int=3) -> None:
    return None

@type_enforced.Enforcer
def my_fn_args(a: int , *args, b: [int, str]=2, c: int=3) -> None:
    return None

@type_enforced.Enforcer
def my_fn_kwargs(a: int, b: [int, str]=2, c: int=3, **kwargs) -> None:
    return None

@type_enforced.Enforcer
def my_fn_args_kwargs(a: int , *args, b: [int, str]=2, c: int=3, **kwargs) -> None:
    return None

success=True
success_type_error=True
try:
    my_fn(a=1, b=2, c=3) # No Error
    my_fn_args(a=1, b=2, c=3) # No Error
    my_fn_kwargs(a=1, b=2, c=3) # No Error
    my_fn_args_kwargs(a=1, b=2, c=3) # No Error
except:
    success=False

try:
    my_fn(a='a', b=2, c=3) # Error (a can only accept int)
    success=False
except Exception as e:
    if type(e) != TypeError:
        success_type_error=False

try:
    my_fn_args(a='a', b=2, c=3) # Error (a can only accept int)
    success=False
except Exception as e:
    if type(e) != TypeError:
        success_type_error=False

try:
    my_fn_kwargs(a='a', b=2, c=3) # Error (a can only accept int)
    success=False
except Exception as e:
    if type(e) != TypeError:
        success_type_error=False

try:
    my_fn_args_kwargs(a='a', b=2, c=3) # Error (a can only accept int)
    success=False
except Exception as e:
    if type(e) != TypeError:
        success_type_error=False



if success and success_type_error:
    print('test_fn_5.py passed')
else:
    print('test_fn_5.py failed')
